using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.BusinessLogicsContracts;
using PrecastConcretePlantContracts.SearchModels;
using PrecastConcretePlantContracts.StoragesContracts;
using PrecastConcretePlantContracts.ViewModels;
using Microsoft.Extensions.Logging;
using PrecastConcretePlantDataModels.Enums;

namespace PrecastConcretePlantBusinessLogic.BusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        private readonly IReinforcedStorage _reinforcedStorage;
        private readonly IShopLogic _shopLogic;

        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IReinforcedStorage reinforcedStorage, IShopLogic shopLogic)
        {
            _logger = logger;
            _shopLogic = shopLogic;
            _reinforcedStorage = reinforcedStorage;
            _orderStorage = orderStorage;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                throw new ArgumentException(
                    $"Статус заказа должен быть {OrderStatus.Неизвестен}", nameof(model));
            }
            model.Status = OrderStatus.Принят;
            model.DateCreate = DateTime.Now;
            if (_orderStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool TakeOrderInWork(OrderBindingModel model) => SetOrderStatus(model, OrderStatus.Выполняется);
        public bool DeliveryOrder(OrderBindingModel model) => SetOrderStatus(model, OrderStatus.Выдан);
        public bool FinishOrder(OrderBindingModel model)
        {
            model.DateImplement = DateTime.Now;
            return SetOrderStatus(model, OrderStatus.Готов);
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. OrderName.Id:{ Id} ", model?.Id);
            var list = (model == null) ? _orderStorage.GetFullList() :
            _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        private bool CheckModel(OrderBindingModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentException("Количество изделий в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentException("Суммарная стоимость заказа должна быть больше 0", nameof(model.Sum));
            }
            if (model.DateCreate > model.DateImplement)
            {
                throw new ArgumentException("Время создания заказа не может быть больше времени его выполнения", nameof(model.DateImplement));
            }
            return true;
        }

        private bool SetOrderStatus(OrderBindingModel model, OrderStatus orderStatus)
        {
            var vmodel = _orderStorage.GetElement(new() { Id = model.Id });
            if (vmodel == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if ((int)vmodel.Status + 1 != (int)orderStatus)
            {
                throw new ArgumentException($"Попытка перевести заказ не в следующий статус: " +
                    $"Текущий статус: {vmodel.Status} \n" +
                    $"Планируемый статус: {orderStatus} \n" +
                    $"Доступный статус: {(OrderStatus)((int)vmodel.Status + 1)}",
                    nameof(vmodel));
            }
            if (orderStatus == OrderStatus.Готов)
            {
                var vreinforced = _reinforcedStorage.GetElement(new() { Id = vmodel.ReinforcedId });

                if (vreinforced == null || !_shopLogic.AddReinforcediesInShops(vreinforced, vmodel.Count))
                {
                    throw new Exception($"Не удалось заполнить магазины изделием '{vreinforced?.ReinforcedName ?? string.Empty}' из заказа {vmodel.Id}");
                }
            }
            model.Status = orderStatus;
            model.DateCreate = vmodel.DateCreate;
            if (model.DateImplement == null)
                model.DateImplement = vmodel.DateImplement;
            model.ReinforcedId = vmodel.ReinforcedId;
            model.Sum = vmodel.Sum;
            model.Count = vmodel.Count;
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
    }
}
